module palette_mixer (

	// inputs
	input clk,
	input vs_in,
	input hs_in,
	input hde_in,
	input vde_in,
	input [3:0] pc_ena_in,
	input [7:0] txt_pixel_in,
	input [7:0] gfx_pixel_in,
	
	// outputs
	output reg [3:0] pc_ena_out,
	//
	output reg [7:0] raster_out_r,
	output reg [7:0] raster_out_g,
	output reg [7:0] raster_out_b,
	//
	output reg [7:0] data_out_b,
	output reg vs_out,
	output reg hs_out,
	output reg hde_out,
	output reg vde_out

);

parameter PIPE_DELAY = 6;	// This parameter selects the number of pixel clocks to delay the VDE and sync outputs.  Only use 2 through 9.

wire pc_enable = (pc_ena_in[3:0]==0);

wire [15:0] txt_pixel_out;
wire [3:0] alpha_blend;
wire [7:0] text_r;
wire [7:0] text_g;
wire [7:0] text_b;

wire [15:0] gfx_pixel_out;
wire [7:0] graphics_r;
wire [7:0] graphics_g;
wire [7:0] graphics_b;

reg [9:0] hde_pipe, vde_pipe, hs_pipe, vs_pipe;	// passthru delay pipes

assign alpha_blend = txt_pixel_out[15:12];
assign text_r[7:4] = txt_pixel_out[11:8];
assign text_r[3:0] = txt_pixel_out[11:8];
assign text_g[7:4] = txt_pixel_out[7:4];
assign text_g[3:0] = txt_pixel_out[7:4];
assign text_b[7:4] = txt_pixel_out[3:0];
assign text_b[3:0] = txt_pixel_out[3:0];

assign graphics_r[7:3] = gfx_pixel_out[15:11];
assign graphics_r[2:0] = gfx_pixel_out[15:13];
assign graphics_g[7:2] = gfx_pixel_out[10:5];
assign graphics_g[1:0] = gfx_pixel_out[10:9];
assign graphics_b[7:3] = gfx_pixel_out[4:0];
assign graphics_b[2:0] = gfx_pixel_out[4:2];

// *********************************************************************
// *
// * create a text/sprite palette RAM instance (4444)
// *
// *********************************************************************
dual_port_palette_ram_INTEL text_palette_RAM(

	// inputs
	.clock_a(clk),
	.clock_b(clk),
	.pc_ena_in(pc_ena_in),
	.pixel_in(txt_pixel_in),
	.address_b(4'h0000),
	.data_a(2'h00),
	.data_b(2'h00),
	.enable_a(pc_enable),
	.enable_b(pc_enable),
	.wren_a(1'b0),
	.wren_b(1'b0),
	
	// outputs
	.pc_ena_out(pc_ena_out),
	.pixel_out(txt_pixel_out),
	.data_out_b(data_out_b),
	
	// registered outputs
	.addr_out_a()

);

defparam text_palette_RAM.init_file = "palette_4444.mif";

// *********************************************************************
// *
// * create a graphics palette RAM instance (565)
// *
// *********************************************************************
dual_port_palette_ram_INTEL graphics_palette_RAM(

	// inputs
	.clock_a(clk),
	.clock_b(clk),
	.pc_ena_in(pc_ena_in),
	.pixel_in(gfx_pixel_in),
	.address_b(4'h0000),
	.data_a(2'h00),
	.data_b(2'h00),
	.enable_a(pc_enable),
	.enable_b(pc_enable),
	.wren_a(1'b0),
	.wren_b(1'b0),
	
	// outputs
	.pc_ena_out(pc_ena_out),
	.pixel_out(gfx_pixel_out),
	.data_out_b(data_out_b),
	
	// registered outputs
	.addr_out_a()

);

defparam graphics_palette_RAM.init_file = "palette_565.mif";

// *********************************************************************

always @(posedge clk) begin

	if (pc_ena_in[3:0] == 0) begin
		
		hde_pipe[0]		<= hde_in;
		hde_pipe[9:1]	<= hde_pipe[8:0];
		hde_out			<= hde_pipe[PIPE_DELAY-1];
		
		vde_pipe[0]		<= vde_in;
		vde_pipe[9:1]	<= vde_pipe[8:0];
		vde_out			<= vde_pipe[PIPE_DELAY-1];
		
		hs_pipe[0]		<= hs_in;
		hs_pipe[9:1]	<= hs_pipe[8:0];
		hs_out			<= hs_pipe[PIPE_DELAY-1];
		
		vs_pipe[0]		<= vs_in;
		vs_pipe[9:1]	<= vs_pipe[8:0];
		vs_out			<= vs_pipe[PIPE_DELAY-1];
		
		// mix output rgb
		raster_out_r	<= ((text_r[7:0] * ~alpha_blend[3:0]) + (graphics_r[7:0] * alpha_blend[3:0])) >> 4;
		raster_out_g	<= ((text_g[7:0] * ~alpha_blend[3:0]) + (graphics_g[7:0] * alpha_blend[3:0])) >> 4;
		raster_out_b	<= ((text_b[7:0] * ~alpha_blend[3:0]) + (graphics_b[7:0] * alpha_blend[3:0])) >> 4;
		
	end

end

endmodule
